@@ -22,6 +22,7 @@
 #include "modulemd-profile.h"
 #include "modulemd-rpm-map-entry.h"
 #include "modulemd-service-level.h"
+#include "modulemd-obsoletes.h"
 #include <glib-object.h>
 
 G_BEGIN_DECLS
@@ -258,6 +259,23 @@ const gchar *
 modulemd_module_stream_v2_get_tracker (ModulemdModuleStreamV2 *self);
 
 
+/**
+ * modulemd_module_stream_v2_get_obsoletes_resolved:
+ * @self: (in): This #ModulemdModuleStreamV2 object.
+ *
+ * Returns: (transfer none): The #ModulemdObsoletes information associated with
+ * this object. If the associated obsoletes has reset attribute set, this
+ * function doesn't return it. From outside obsoletes with reset looks like
+ * there is no obsoletes set for this stream. Every obsoletes (even with reset) can
+ * be accessed from the streams module.
+ *
+ * Since: 2.10
+ */
+ModulemdObsoletes *
+modulemd_module_stream_v2_get_obsoletes_resolved (
+  ModulemdModuleStreamV2 *self);
+
+
 /* ===== Non-property Methods ===== */
 
 
@@ -886,4 +904,42 @@ GVariant *
 modulemd_module_stream_v2_get_xmd (ModulemdModuleStreamV2 *self);
 
 
+/**
+ * modulemd_module_stream_v2_set_static_context:
+ * @self: (in): This #ModulemdModuleStreamV2 object.
+ *
+ * Marks this #ModulemdModuleStreamV2 to indicate to dependency solvers that
+ * they must treat the `context` attribute as informative.
+ *
+ * Since: 2.11
+ */
+void
+modulemd_module_stream_v2_set_static_context (ModulemdModuleStreamV2 *self);
+
+
+/**
+ * modulemd_module_stream_v2_unset_static_context:
+ * @self: (in): This #ModulemdModuleStreamV2 object.
+ *
+ * Marks this #ModulemdModuleStreamV2 to indicate to dependency solvers that
+ * they must ignore the `context` attribute as a uniqueness element.
+ *
+ * Since: 2.11
+ */
+void
+modulemd_module_stream_v2_unset_static_context (ModulemdModuleStreamV2 *self);
+
+
+/**
+ * modulemd_module_stream_v2_is_static_context:
+ * @self: (in): This #ModulemdModuleStreamV2 object.
+ *
+ * Returns: Whether the `context` attribute should be treated as static.
+ *
+ * Since: 2.11
+ */
+gboolean
+modulemd_module_stream_v2_is_static_context (ModulemdModuleStreamV2 *self);
+
+
 G_END_DECLS
