@@ -26,7 +26,6 @@
 #include "private/modulemd-component-module-private.h"
 #include "private/modulemd-component-private.h"
 #include "private/modulemd-component-rpm-private.h"
-#include "private/modulemd-dependencies-private.h"
 #include "private/modulemd-module-stream-private.h"
 #include "private/modulemd-module-stream-v1-private.h"
 #include "private/modulemd-profile-private.h"
@@ -1533,9 +1532,6 @@ modulemd_module_stream_v1_parse_components (
   gboolean strict,
   GError **error);
 
-static GVariant *
-modulemd_module_stream_v1_parse_raw (yaml_parser_t *parser, GError **error);
-
 
 ModulemdModuleStreamV1 *
 modulemd_module_stream_v1_parse_yaml (ModulemdSubdocumentInfo *subdoc,
@@ -1692,8 +1688,7 @@ modulemd_module_stream_v1_parse_yaml (ModulemdSubdocumentInfo *subdoc,
           /* Extensible Metadata */
           else if (g_str_equal ((const gchar *)event.data.scalar.value, "xmd"))
             {
-              xmd =
-                modulemd_module_stream_v1_parse_raw (&parser, &nested_error);
+              xmd = mmd_parse_xmd (&parser, &nested_error);
               if (!xmd)
                 {
                   g_propagate_error (error, g_steal_pointer (&nested_error));
@@ -2425,7 +2420,6 @@ modulemd_module_stream_v1_parse_rpm_components (
             parser,
             (const gchar *)event.data.scalar.value,
             strict,
-            FALSE,
             &nested_error);
           if (!component)
             {
@@ -2517,45 +2511,6 @@ modulemd_module_stream_v1_parse_module_components (
 }
 
 
-static GVariant *
-modulemd_module_stream_v1_parse_raw (yaml_parser_t *parser, GError **error)
-{
-  MODULEMD_INIT_TRACE ();
-  MMD_INIT_YAML_EVENT (event);
-  g_autoptr (GVariant) variant = NULL;
-  g_autoptr (GError) nested_error = NULL;
-
-  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-
-  YAML_PARSER_PARSE_WITH_EXIT (parser, &event, error);
-
-  switch (event.type)
-    {
-    case YAML_SCALAR_EVENT:
-      variant =
-        mmd_variant_from_scalar ((const gchar *)event.data.scalar.value);
-      if (!variant)
-        {
-          MMD_YAML_ERROR_EVENT_EXIT (error, event, "Error parsing scalar");
-        }
-      break;
-
-    case YAML_MAPPING_START_EVENT:
-      variant = mmd_variant_from_mapping (parser, &nested_error);
-      break;
-
-    default:
-      MMD_YAML_ERROR_EVENT_EXIT (error,
-                                 event,
-                                 "Unexpected YAML event in raw parsing: %s",
-                                 mmd_yaml_get_event_name (event.type));
-      break;
-    }
-
-  return g_steal_pointer (&variant);
-}
-
-
 gboolean
 modulemd_module_stream_v1_emit_yaml (ModulemdModuleStreamV1 *self,
                                      yaml_emitter_t *emitter,
