@@ -49,6 +49,7 @@ enum
   PROP_COMMUNITY,
   PROP_DOCUMENTATION,
   PROP_TRACKER,
+  PROP_STATIC_CONTEXT,
   N_PROPS
 };
 
@@ -101,6 +102,8 @@ modulemd_module_stream_v2_finalize (GObject *object)
 
   g_clear_pointer (&self->dependencies, g_ptr_array_unref);
 
+  g_clear_pointer (&self->obsoletes, g_object_unref);
+
   g_clear_pointer (&self->xmd, g_variant_unref);
 
   G_OBJECT_CLASS (modulemd_module_stream_v2_parent_class)->finalize (object);
@@ -151,6 +154,14 @@ modulemd_module_stream_v2_equals (ModulemdModuleStream *self_1,
       return FALSE;
     }
 
+  /* Test the negations of static_context just in case somehow they are
+   * different non-zero values
+   */
+  if (!v2_self_1->static_context != !v2_self_2->static_context)
+    {
+      return FALSE;
+    }
+
   if (!modulemd_buildopts_equals (v2_self_1->buildopts, v2_self_2->buildopts))
     {
       return FALSE;
@@ -296,7 +307,10 @@ modulemd_module_stream_v2_set_buildopts (ModulemdModuleStreamV2 *self,
   g_return_if_fail (MODULEMD_IS_MODULE_STREAM_V2 (self));
 
   g_clear_object (&self->buildopts);
-  self->buildopts = modulemd_buildopts_copy (buildopts);
+  if (buildopts)
+    {
+      self->buildopts = modulemd_buildopts_copy (buildopts);
+    }
 
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_BUILDOPTS]);
 }
@@ -436,6 +450,42 @@ modulemd_module_stream_v2_get_tracker (ModulemdModuleStreamV2 *self)
 }
 
 
+ModulemdObsoletes *
+modulemd_module_stream_v2_get_obsoletes_resolved (ModulemdModuleStreamV2 *self)
+{
+  g_return_val_if_fail (MODULEMD_IS_MODULE_STREAM_V2 (self), NULL);
+
+  ModulemdObsoletes *o = self->obsoletes;
+  if (o && modulemd_obsoletes_get_reset (o))
+    {
+      return NULL;
+    }
+
+  return o;
+}
+
+void
+modulemd_module_stream_v2_associate_obsoletes (ModulemdModuleStreamV2 *self,
+                                               ModulemdObsoletes *obsoletes)
+{
+  g_return_if_fail (MODULEMD_IS_MODULE_STREAM_V2 (self));
+
+  g_clear_pointer (&self->obsoletes, g_object_unref);
+  if (obsoletes != NULL)
+    {
+      self->obsoletes = g_object_ref (obsoletes);
+    }
+}
+
+ModulemdObsoletes *
+modulemd_module_stream_v2_get_obsoletes (ModulemdModuleStreamV2 *self)
+{
+  g_return_val_if_fail (MODULEMD_IS_MODULE_STREAM_V2 (self), NULL);
+
+  return self->obsoletes;
+}
+
+
 /* ===== Non-property Methods ===== */
 
 void
@@ -1158,6 +1208,72 @@ modulemd_module_stream_v2_includes_nevra (ModulemdModuleStreamV2 *self,
 }
 
 
+void
+modulemd_module_stream_v2_set_static_context (ModulemdModuleStreamV2 *self)
+{
+  self->static_context = TRUE;
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_STATIC_CONTEXT]);
+}
+
+
+void
+modulemd_module_stream_v2_unset_static_context (ModulemdModuleStreamV2 *self)
+{
+  self->static_context = FALSE;
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_STATIC_CONTEXT]);
+}
+
+
+gboolean
+modulemd_module_stream_v2_is_static_context (ModulemdModuleStreamV2 *self)
+{
+  return self->static_context;
+}
+
+
+static gboolean
+modulemd_module_stream_v2_validate_context (const gchar *context,
+                                            GError **error)
+{
+  /* must be string of up to MMD_MAXCONTEXTLEN [a-zA-Z0-9] */
+
+  if (context == NULL || *context == '\0')
+    {
+      g_set_error (
+        error, MODULEMD_ERROR, MMD_ERROR_VALIDATE, "Empty stream context");
+      return FALSE;
+    }
+
+  if (strlen (context) > MMD_MAXCONTEXTLEN)
+    {
+      g_set_error (error,
+                   MODULEMD_ERROR,
+                   MMD_ERROR_VALIDATE,
+                   "Stream context '%s' exceeds maximum length (%d)",
+                   context,
+                   MMD_MAXCONTEXTLEN);
+      return FALSE;
+    }
+
+  for (const gchar *i = context; *i != '\0'; i++)
+    {
+      if (!g_ascii_isalnum (*i))
+        {
+          g_set_error (error,
+                       MODULEMD_ERROR,
+                       MMD_ERROR_VALIDATE,
+                       "Non-alphanumeric character in stream context '%s'",
+                       context);
+          return FALSE;
+        }
+    }
+
+  return TRUE;
+}
+
+
 static gboolean
 modulemd_module_stream_v2_validate (ModulemdModuleStream *self, GError **error)
 {
@@ -1169,6 +1285,7 @@ modulemd_module_stream_v2_validate (ModulemdModuleStream *self, GError **error)
   ModulemdDependencies *deps = NULL;
   g_autoptr (GError) nested_error = NULL;
   g_auto (GStrv) buildopts_arches = NULL;
+  const gchar *context = NULL;
 
   g_return_val_if_fail (MODULEMD_IS_MODULE_STREAM_V2 (self), FALSE);
   v2_self = MODULEMD_MODULE_STREAM_V2 (self);
@@ -1179,6 +1296,22 @@ modulemd_module_stream_v2_validate (ModulemdModuleStream *self, GError **error)
       return FALSE;
     }
 
+  /* Validate static context if present */
+  if (v2_self->static_context)
+    {
+      context =
+        modulemd_module_stream_get_context (MODULEMD_MODULE_STREAM (self));
+      if (context)
+        {
+          if (!modulemd_module_stream_v2_validate_context (context,
+                                                           &nested_error))
+            {
+              g_propagate_error (error, g_steal_pointer (&nested_error));
+              return FALSE;
+            }
+        }
+    }
+
   /* Make sure that mandatory fields are present */
   if (!modulemd_module_stream_v2_get_summary (v2_self, "C"))
     {
@@ -1301,6 +1434,11 @@ modulemd_module_stream_v2_get_property (GObject *object,
       g_value_set_string (value, modulemd_module_stream_v2_get_tracker (self));
       break;
 
+    case PROP_STATIC_CONTEXT:
+      g_value_set_boolean (value,
+                           modulemd_module_stream_v2_is_static_context (self));
+      break;
+
     default: G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
 }
@@ -1337,6 +1475,17 @@ modulemd_module_stream_v2_set_property (GObject *object,
       modulemd_module_stream_v2_set_tracker (self, g_value_get_string (value));
       break;
 
+    case PROP_STATIC_CONTEXT:
+      if (g_value_get_boolean (value))
+        {
+          modulemd_module_stream_v2_set_static_context (self);
+        }
+      else
+        {
+          modulemd_module_stream_v2_unset_static_context (self);
+        }
+      break;
+
     default: G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     }
 }
@@ -1393,6 +1542,7 @@ modulemd_module_stream_v2_copy (ModulemdModuleStream *self,
   STREAM_COPY_IF_SET (v2, copy, v2_self, documentation);
   STREAM_COPY_IF_SET_WITH_LOCALE (v2, copy, v2_self, summary);
   STREAM_COPY_IF_SET (v2, copy, v2_self, tracker);
+  copy->static_context = v2_self->static_context;
 
   /* Internal Data Structures: With replace function */
   STREAM_REPLACE_HASHTABLE (v2, copy, v2_self, content_licenses);
@@ -1417,6 +1567,9 @@ modulemd_module_stream_v2_copy (ModulemdModuleStream *self,
 
   STREAM_COPY_IF_SET (v2, copy, v2_self, xmd);
 
+  modulemd_module_stream_v2_associate_obsoletes (
+    copy, modulemd_module_stream_v2_get_obsoletes (v2_self));
+
   return MODULEMD_MODULE_STREAM (g_steal_pointer (&copy));
 }
 
@@ -1536,6 +1689,13 @@ modulemd_module_stream_v2_class_init (ModulemdModuleStreamV2Class *klass)
     NULL,
     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT);
 
+  properties[PROP_STATIC_CONTEXT] =
+    g_param_spec_boolean ("static-context",
+                          "Static Context",
+                          "Whether the context is static",
+                          0,
+                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+
   g_object_class_install_properties (object_class, N_PROPS, properties);
 }
 
@@ -1618,7 +1778,6 @@ modulemd_module_stream_v2_parse_components (
   yaml_parser_t *parser,
   ModulemdModuleStreamV2 *modulestream,
   gboolean strict,
-  gboolean only_packager,
   GError **error);
 
 static gboolean
@@ -1628,9 +1787,6 @@ modulemd_module_stream_v2_parse_artifacts (
   gboolean strict,
   GError **error);
 
-static GVariant *
-modulemd_module_stream_v2_parse_raw (yaml_parser_t *parser, GError **error);
-
 
 ModulemdModuleStreamV2 *
 modulemd_module_stream_v2_parse_yaml (ModulemdSubdocumentInfo *subdoc,
@@ -1648,6 +1804,7 @@ modulemd_module_stream_v2_parse_yaml (ModulemdSubdocumentInfo *subdoc,
   g_autoptr (ModulemdBuildopts) buildopts = NULL;
   g_autoptr (GVariant) xmd = NULL;
   guint64 version;
+  gboolean static_context;
 
   if (!modulemd_subdocument_info_get_data_parser (
         subdoc, &parser, strict, error))
@@ -1730,6 +1887,28 @@ modulemd_module_stream_v2_parse_yaml (ModulemdSubdocumentInfo *subdoc,
                 MODULEMD_MODULE_STREAM (modulestream));
             }
 
+          else if (g_str_equal ((const gchar *)event.data.scalar.value,
+                                "static_context"))
+            {
+              static_context =
+                modulemd_yaml_parse_bool (&parser, &nested_error);
+              if (nested_error)
+                {
+                  g_propagate_error (error, g_steal_pointer (&nested_error));
+                  return NULL;
+                }
+
+              if (static_context)
+                {
+                  modulemd_module_stream_v2_set_static_context (modulestream);
+                }
+              else
+                {
+                  modulemd_module_stream_v2_unset_static_context (
+                    modulestream);
+                }
+            }
+
           /* Module Artifact Architecture */
           else if (g_str_equal ((const gchar *)event.data.scalar.value,
                                 "arch") &&
@@ -1796,8 +1975,7 @@ modulemd_module_stream_v2_parse_yaml (ModulemdSubdocumentInfo *subdoc,
                                 "xmd") &&
                    !only_packager)
             {
-              xmd =
-                modulemd_module_stream_v2_parse_raw (&parser, &nested_error);
+              xmd = mmd_parse_xmd (&parser, &nested_error);
               if (!xmd)
                 {
                   g_propagate_error (error, g_steal_pointer (&nested_error));
@@ -1885,11 +2063,8 @@ modulemd_module_stream_v2_parse_yaml (ModulemdSubdocumentInfo *subdoc,
           else if (g_str_equal ((const gchar *)event.data.scalar.value,
                                 "components"))
             {
-              if (!modulemd_module_stream_v2_parse_components (&parser,
-                                                               modulestream,
-                                                               strict,
-                                                               only_packager,
-                                                               &nested_error))
+              if (!modulemd_module_stream_v2_parse_components (
+                    &parser, modulestream, strict, &nested_error))
                 {
                   g_propagate_error (error, g_steal_pointer (&nested_error));
                   return NULL;
@@ -2348,7 +2523,6 @@ modulemd_module_stream_v2_parse_rpm_components (
   yaml_parser_t *parser,
   ModulemdModuleStreamV2 *modulestream,
   gboolean strict,
-  gboolean only_packager,
   GError **error);
 static gboolean
 modulemd_module_stream_v2_parse_module_components (
@@ -2363,7 +2537,6 @@ modulemd_module_stream_v2_parse_components (
   yaml_parser_t *parser,
   ModulemdModuleStreamV2 *modulestream,
   gboolean strict,
-  gboolean only_packager,
   GError **error)
 {
   MODULEMD_INIT_TRACE ();
@@ -2397,11 +2570,7 @@ modulemd_module_stream_v2_parse_components (
           if (g_str_equal ((const gchar *)event.data.scalar.value, "rpms"))
             {
               if (!modulemd_module_stream_v2_parse_rpm_components (
-                    parser,
-                    modulestream,
-                    strict,
-                    only_packager,
-                    &nested_error))
+                    parser, modulestream, strict, &nested_error))
                 {
                   g_propagate_error (error, g_steal_pointer (&nested_error));
                   return FALSE;
@@ -2450,7 +2619,6 @@ modulemd_module_stream_v2_parse_rpm_components (
   yaml_parser_t *parser,
   ModulemdModuleStreamV2 *modulestream,
   gboolean strict,
-  gboolean only_packager,
   GError **error)
 {
   MODULEMD_INIT_TRACE ();
@@ -2485,7 +2653,6 @@ modulemd_module_stream_v2_parse_rpm_components (
             parser,
             (const gchar *)event.data.scalar.value,
             strict,
-            only_packager,
             &nested_error);
           if (!component)
             {
@@ -2801,45 +2968,6 @@ modulemd_module_stream_v2_parse_rpm_map_digest (
 }
 
 
-static GVariant *
-modulemd_module_stream_v2_parse_raw (yaml_parser_t *parser, GError **error)
-{
-  MODULEMD_INIT_TRACE ();
-  MMD_INIT_YAML_EVENT (event);
-  GVariant *variant = NULL;
-  g_autoptr (GError) nested_error = NULL;
-
-  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-
-  YAML_PARSER_PARSE_WITH_EXIT (parser, &event, error);
-
-  switch (event.type)
-    {
-    case YAML_SCALAR_EVENT:
-      variant =
-        mmd_variant_from_scalar ((const gchar *)event.data.scalar.value);
-      if (!variant)
-        {
-          MMD_YAML_ERROR_EVENT_EXIT (error, event, "Error parsing scalar");
-        }
-      break;
-
-    case YAML_MAPPING_START_EVENT:
-      variant = mmd_variant_from_mapping (parser, &nested_error);
-      break;
-
-    default:
-      MMD_YAML_ERROR_EVENT_EXIT (error,
-                                 event,
-                                 "Unexpected YAML event in raw parsing: %s",
-                                 mmd_yaml_get_event_name (event.type));
-      break;
-    }
-
-  return g_variant_ref_sink (variant);
-}
-
-
 gboolean
 modulemd_module_stream_v2_emit_rpm_map (ModulemdModuleStreamV2 *self,
                                         yaml_emitter_t *emitter,
@@ -2858,6 +2986,11 @@ modulemd_module_stream_v2_emit_yaml (ModulemdModuleStreamV2 *self,
       return FALSE;
     }
 
+  if (modulemd_module_stream_v2_is_static_context (self))
+    {
+      EMIT_KEY_VALUE (emitter, error, "static_context", "true");
+    }
+
   EMIT_KEY_VALUE_IF_SET (
     emitter, error, "arch", modulemd_module_stream_v2_get_arch (self));
   EMIT_KEY_VALUE (emitter, error, "summary", self->summary);
