@@ -1,6 +1,6 @@
 /*
  * This file is part of libmodulemd
- * Copyright (C) 2018 Red Hat, Inc.
+ * Copyright (C) 2018-2020 Red Hat, Inc.
  *
  * Fedora-License-Identifier: MIT
  * SPDX-2.0-License-Identifier: MIT
@@ -11,14 +11,20 @@
  * For more information on free software, see <https://www.gnu.org/philosophy/free-sw.en.html>.
  */
 
+#include "modulemd.h"
 #include "modulemd-errors.h"
+#include "modulemd-module-index.h"
 #include "modulemd-module-stream-v1.h"
 #include "modulemd-module-stream-v2.h"
 #include "modulemd-module-stream.h"
+#include "modulemd-packager-v3.h"
+#include "private/glib-extensions.h"
 #include "private/modulemd-component-private.h"
+#include "private/modulemd-module-private.h"
 #include "private/modulemd-module-stream-private.h"
 #include "private/modulemd-module-stream-v1-private.h"
 #include "private/modulemd-module-stream-v2-private.h"
+#include "private/modulemd-packager-v3-private.h"
 #include "private/modulemd-subdocument-info-private.h"
 #include "private/modulemd-util.h"
 #include "private/modulemd-yaml.h"
@@ -162,6 +168,7 @@ modulemd_module_stream_read_yaml (yaml_parser_t *parser,
   g_autoptr (GError) nested_error = NULL;
   g_autoptr (ModulemdModuleStream) stream = NULL;
   g_autoptr (ModulemdSubdocumentInfo) subdoc = NULL;
+  g_autoptr (ModulemdPackagerV3) packager_v3 = NULL;
   ModulemdYamlDocumentTypeEnum doctype;
   const GError *gerror = NULL;
 
@@ -252,6 +259,38 @@ modulemd_module_stream_read_yaml (yaml_parser_t *parser,
         }
       break;
 
+    case MD_PACKAGER_VERSION_THREE:
+      if (doctype == MODULEMD_YAML_DOC_PACKAGER)
+        {
+          packager_v3 =
+            modulemd_packager_v3_parse_yaml (subdoc, &nested_error);
+          if (!packager_v3)
+            {
+              g_propagate_error (error, g_steal_pointer (&nested_error));
+              return NULL;
+            }
+
+          stream = MODULEMD_MODULE_STREAM (
+            modulemd_packager_v3_to_stream_v2 (packager_v3, &nested_error));
+          if (!stream)
+            {
+              g_propagate_error (error, g_steal_pointer (&nested_error));
+              return NULL;
+            }
+
+          g_clear_object (&packager_v3);
+        }
+      else
+        {
+          g_set_error (error,
+                       MODULEMD_YAML_ERROR,
+                       MMD_YAML_ERROR_PARSE,
+                       "Invalid ModuleStream version: %" PRIu64,
+                       modulemd_subdocument_info_get_mdversion (subdoc));
+          return NULL;
+        }
+      break;
+
     default:
       g_set_error (error,
                    MODULEMD_YAML_ERROR,
@@ -313,7 +352,7 @@ modulemd_module_stream_finalize (GObject *object)
   g_clear_pointer (&priv->stream_name, g_free);
   g_clear_pointer (&priv->context, g_free);
   g_clear_pointer (&priv->arch, g_free);
-  g_clear_pointer (&priv->translation, g_object_unref);
+  g_clear_object (&priv->translation);
 
   G_OBJECT_CLASS (modulemd_module_stream_parent_class)->finalize (object);
 }
@@ -455,9 +494,6 @@ modulemd_module_stream_copy (ModulemdModuleStream *self,
 }
 
 
-static ModulemdModuleStream *
-modulemd_module_stream_upgrade_to_v2 (ModulemdModuleStream *from);
-
 ModulemdModuleStream *
 modulemd_module_stream_upgrade (ModulemdModuleStream *self,
                                 guint64 mdversion,
@@ -465,6 +501,7 @@ modulemd_module_stream_upgrade (ModulemdModuleStream *self,
 {
   g_autoptr (ModulemdModuleStream) current_stream = NULL;
   g_autoptr (ModulemdModuleStream) updated_stream = NULL;
+  g_autoptr (GError) nested_error = NULL;
   guint64 current_mdversion = modulemd_module_stream_get_mdversion (self);
 
   g_return_val_if_fail (MODULEMD_IS_MODULE_STREAM (self), NULL);
@@ -498,13 +535,13 @@ modulemd_module_stream_upgrade (ModulemdModuleStream *self,
       switch (current_mdversion)
         {
         case 1:
-          /* Upgrade to ModuleStreamV2 */
+          /* Upgrade from ModuleStreamV1 to ModuleStreamV2 */
           updated_stream =
-            modulemd_module_stream_upgrade_to_v2 (current_stream);
+            modulemd_module_stream_upgrade_v1_to_v2 (current_stream);
           if (!updated_stream)
             {
               /* This should be impossible, since there are no failure returns
-               * from modulemd_module_stream_upgrade_to_v2()
+               * from modulemd_module_stream_upgrade_v1_to_v2()
                */
               g_set_error (error,
                            MODULEMD_ERROR,
@@ -534,9 +571,118 @@ modulemd_module_stream_upgrade (ModulemdModuleStream *self,
   return g_steal_pointer (&current_stream);
 }
 
+ModulemdModule *
+modulemd_module_stream_upgrade_ext (ModulemdModuleStream *self,
+                                    guint64 mdversion,
+                                    GError **error)
+{
+  g_autoptr (ModulemdModuleStream) current_stream = NULL;
+  g_autoptr (ModulemdModuleStream) updated_stream = NULL;
+  g_autoptr (ModulemdModule) current_module = NULL;
+  g_autoptr (ModulemdModule) updated_module = NULL;
+  g_autoptr (GError) nested_error = NULL;
+  guint64 current_mdversion = modulemd_module_stream_get_mdversion (self);
 
-static ModulemdModuleStream *
-modulemd_module_stream_upgrade_to_v2 (ModulemdModuleStream *from)
+  g_return_val_if_fail (MODULEMD_IS_MODULE_STREAM (self), NULL);
+
+  if (!mdversion)
+    {
+      /* If target mdversion is unspecified, set it to the latest */
+      mdversion = MD_MODULESTREAM_VERSION_LATEST;
+    }
+
+  if (mdversion < current_mdversion)
+    {
+      /* Downgrades are not supported */
+      g_set_error_literal (error,
+                           MODULEMD_ERROR,
+                           MMD_ERROR_UPGRADE,
+                           "ModuleStream downgrades are not supported.");
+      return NULL;
+    }
+
+  if (current_mdversion == mdversion)
+    {
+      /* If we're already at the requested version, just wrap it in a Module
+       * and return that.
+       */
+      current_module =
+        modulemd_module_new (modulemd_module_stream_get_module_name (self));
+
+      if (modulemd_module_add_stream (
+            current_module, self, current_mdversion, &nested_error) ==
+          MD_MODULESTREAM_VERSION_ERROR)
+        {
+          g_propagate_error (error, g_steal_pointer (&nested_error));
+          return NULL;
+        }
+
+      return g_steal_pointer (&current_module);
+    }
+
+  current_stream = g_object_ref (self);
+
+  while (current_mdversion != mdversion)
+    {
+      switch (current_mdversion)
+        {
+        case 1:
+          /* Upgrade from ModuleStreamV1 to ModuleStreamV2 */
+          updated_stream =
+            modulemd_module_stream_upgrade_v1_to_v2 (current_stream);
+          if (!updated_stream)
+            {
+              /* This should be impossible, since there are no failure returns
+               * from modulemd_module_stream_upgrade_v1_to_v2()
+               */
+              g_set_error (error,
+                           MODULEMD_ERROR,
+                           MMD_ERROR_UPGRADE,
+                           "Upgrading to v2 failed for an unknown reason");
+              return NULL;
+            }
+          g_object_unref (current_stream);
+          current_stream = g_steal_pointer (&updated_stream);
+          current_mdversion = MD_MODULESTREAM_VERSION_TWO;
+          break;
+
+        default:
+          /* If we get here, it means we failed to address an upgrade. */
+          g_set_error (
+            error,
+            MODULEMD_ERROR,
+            MMD_ERROR_UPGRADE,
+            "Cannot upgrade beyond metadata version %" G_GUINT64_FORMAT,
+            current_mdversion);
+          return NULL;
+        }
+    }
+
+  /* if that latest upgrade was still a Stream, wrap it in a Module */
+  if (current_stream)
+    {
+      current_module = modulemd_module_new (
+        modulemd_module_stream_get_module_name (current_stream));
+
+      if (modulemd_module_add_stream (current_module,
+                                      current_stream,
+                                      current_mdversion,
+                                      &nested_error) ==
+          MD_MODULESTREAM_VERSION_ERROR)
+        {
+          g_propagate_error (error, g_steal_pointer (&nested_error));
+          return NULL;
+        }
+
+      g_clear_object (&current_stream);
+    }
+
+  return g_steal_pointer (&current_module);
+}
+
+
+ModulemdModuleStream *
+modulemd_module_stream_upgrade_v1_to_v2 (ModulemdModuleStream *from)
 {
   ModulemdModuleStreamV1 *v1_stream = NULL;
   g_autoptr (ModulemdModuleStreamV2) copy = NULL;
@@ -729,7 +875,7 @@ modulemd_module_stream_validate_components (GHashTable *components,
        */
       if (has_buildafter && has_buildorder)
         {
-          g_set_error (
+          g_set_error_literal (
             error,
             MODULEMD_ERROR,
             MMD_ERROR_VALIDATE,
@@ -1317,10 +1463,27 @@ modulemd_module_stream_emit_yaml_base (ModulemdModuleStream *self,
   /* Start data: */
   EMIT_MAPPING_START (emitter, error);
 
-  EMIT_KEY_VALUE_IF_SET (
-    emitter, error, "name", modulemd_module_stream_get_module_name (self));
-  EMIT_KEY_VALUE_IF_SET (
-    emitter, error, "stream", modulemd_module_stream_get_stream_name (self));
+  if (modulemd_module_stream_get_module_name (self) != NULL &&
+      !modulemd_module_stream_is_autogen_module_name (self))
+    {
+      EMIT_KEY_VALUE (
+        emitter, error, "name", modulemd_module_stream_get_module_name (self));
+    }
+
+  /* Always emit the stream quoted, since a purely numeric-looking stream such
+   * as 5.30 might otherwise be interpreted by parsers like pyyaml as a number
+   * and result in being read (and written) as '5.3'.
+   */
+  if (modulemd_module_stream_get_stream_name (self) != NULL &&
+      !modulemd_module_stream_is_autogen_stream_name (self))
+    {
+      EMIT_KEY_VALUE_FULL (emitter,
+                           error,
+                           "stream",
+                           modulemd_module_stream_get_stream_name (self),
+                           YAML_DOUBLE_QUOTED_SCALAR_STYLE);
+    }
+
   EMIT_KEY_VALUE_IF_SET (emitter, error, "version", version_string);
   EMIT_KEY_VALUE_IF_SET (
     emitter, error, "context", modulemd_module_stream_get_context (self));
@@ -1390,3 +1553,77 @@ modulemd_module_stream_includes_nevra (ModulemdModuleStream *self,
 
   return FALSE;
 }
+
+void
+modulemd_module_stream_set_autogen_module_name (ModulemdModuleStream *self,
+                                                guint id)
+{
+  g_autofree gchar *name = NULL;
+
+  if (!modulemd_module_stream_get_module_name (self))
+    {
+      name = g_strdup_printf ("__unnamed_module_%u", id);
+      modulemd_module_stream_set_module_name (self, name);
+      g_clear_pointer (&name, g_free);
+    }
+}
+
+void
+modulemd_module_stream_set_autogen_stream_name (ModulemdModuleStream *self,
+                                                guint id)
+{
+  g_autofree gchar *name = NULL;
+
+  if (!modulemd_module_stream_get_stream_name (self))
+    {
+      name = g_strdup_printf ("__unnamed_stream_%u", id);
+      modulemd_module_stream_set_stream_name (self, name);
+      g_clear_pointer (&name, g_free);
+    }
+}
+
+gboolean
+modulemd_module_stream_is_autogen_module_name (ModulemdModuleStream *self)
+{
+  const gchar *name = NULL;
+
+  name = modulemd_module_stream_get_module_name (self);
+  if (name && g_str_has_prefix (name, "__unnamed_module_"))
+    {
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+gboolean
+modulemd_module_stream_is_autogen_stream_name (ModulemdModuleStream *self)
+{
+  const gchar *name = NULL;
+
+  name = modulemd_module_stream_get_stream_name (self);
+  if (name && g_str_has_prefix (name, "__unnamed_stream_"))
+    {
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+void
+modulemd_module_stream_clear_autogen_module_name (ModulemdModuleStream *self)
+{
+  if (modulemd_module_stream_is_autogen_module_name (self))
+    {
+      modulemd_module_stream_set_module_name (self, NULL);
+    }
+}
+
+void
+modulemd_module_stream_clear_autogen_stream_name (ModulemdModuleStream *self)
+{
+  if (modulemd_module_stream_is_autogen_stream_name (self))
+    {
+      modulemd_module_stream_set_stream_name (self, NULL);
+    }
+}
