@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # This file is part of libmodulemd
-# Copyright (C) 2018 Red Hat, Inc.
+# Copyright (C) 2018-2020 Red Hat, Inc.
 #
 # Fedora-License-Identifier: MIT
 # SPDX-2.0-License-Identifier: MIT
@@ -20,6 +20,10 @@ try:
     import unittest
     import gi
 
+    gi._overridesdir = os.path.join(
+        os.getenv("MESON_SOURCE_ROOT"), "bindings", "python", "gi", "overrides"
+    )
+
     gi.require_version("Modulemd", "2.0")
     from gi.repository import GLib
     from gi.repository import Modulemd
@@ -493,6 +497,9 @@ class TestModuleStream(TestBase):
 
     def test_servicelevels(self):
         for version in modulestream_versions:
+            # servicelevels are not supported after v2
+            if version > Modulemd.ModuleStreamVersionEnum.TWO:
+                continue
             stream = Modulemd.ModuleStream.new(version)
             sl = Modulemd.ServiceLevel.new("rawhide")
             sl.set_eol_ymd(1980, 3, 2)
@@ -566,43 +573,59 @@ class TestModuleStream(TestBase):
         self.assertEquals(len(stream.get_dependencies()), 0)
 
     def test_xmd(self):
-        if "_overrides_module" in dir(Modulemd):
-            # The XMD python tests can only be run against the installed lib
-            # because the overrides that translate between python and GVariant
-            # must be installed in /usr/lib/python*/site-packages/gi/overrides
-            # or they are not included when importing Modulemd
-            stream = Modulemd.ModuleStreamV2.new("foo", "bar")
-            # An empty dictionary should be returned if no xmd value is set
-            assert stream.get_xmd() == {}
 
-            xmd = {"outer_key": {"inner_key": ["scalar", "another_scalar"]}}
+        for version in modulestream_versions:
+            # We have a chicken-egg problem with overrides, since they can only
+            # be tested if they are already installed. This means they need to
+            # be run in the CI. In order to avoid changes to these tests or the
+            # overrides breaking things, we'll skip them if the appropriate
+            # override is not installed.
+            if "_overrides_module" in dir(Modulemd) and hasattr(
+                gi.overrides.Modulemd,
+                type(Modulemd.ModuleStream.new(version)).__name__,
+            ):
 
-            stream.set_xmd(xmd)
+                # The XMD python tests can only be run against the installed lib
+                # because the overrides that translate between python and GVariant
+                # must be installed in /usr/lib/python*/site-packages/gi/overrides
+                # or they are not included when importing Modulemd
+                stream = Modulemd.ModuleStream.new(version, "foo", "bar")
+                # An empty dictionary should be returned if no xmd value is set
+                self.assertEqual(stream.get_xmd(), dict())
 
-            xmd_copy = stream.get_xmd()
-            assert xmd_copy
-            assert "outer_key" in xmd_copy
-            assert "inner_key" in xmd_copy["outer_key"]
-            assert "scalar" in xmd_copy["outer_key"]["inner_key"]
-            assert "another_scalar" in xmd_copy["outer_key"]["inner_key"]
+                xmd = {
+                    "outer_key": {"inner_key": ["scalar", "another_scalar"]}
+                }
 
-            # Verify that we can add content and save it back
-            xmd["something"] = ["foo", "bar"]
-            stream.set_xmd(xmd)
+                stream.set_xmd(xmd)
 
-            stream.set_summary("foo")
-            stream.set_description("bar")
-            stream.add_module_license("MIT")
+                xmd_copy = stream.get_xmd()
+                assert xmd_copy
+                assert "outer_key" in xmd_copy
+                assert "inner_key" in xmd_copy["outer_key"]
+                assert "scalar" in xmd_copy["outer_key"]["inner_key"]
+                assert "another_scalar" in xmd_copy["outer_key"]["inner_key"]
 
-            # Verify that we can output the XMD successfully
-            index = Modulemd.ModuleIndex()
-            index.add_module_stream(stream)
-            out_yaml = index.dump_to_string()
+                # Verify that we can add content and save it back
+                xmd["something"] = ["foo", "bar"]
+                stream.set_xmd(xmd)
 
-            self.assertIsNotNone(out_yaml)
+                stream.set_summary("foo")
+                stream.set_description("bar")
+                stream.add_module_license("MIT")
+                if hasattr(stream, "set_platform"):
+                    stream.set_platform("f33")
 
-    def test_upgrade(self):
+                index = Modulemd.ModuleIndex()
+                index.add_module_stream(stream)
+
+                out_yaml = index.dump_to_string()
+
+                self.assertIsNotNone(out_yaml)
+
+    def test_upgrade_v1_to_v2(self):
         v1_stream = Modulemd.ModuleStreamV1.new("SuperModule", "latest")
+        v1_stream.set_context("ctx")
         v1_stream.set_summary("Summary")
         v1_stream.set_description("Description")
         v1_stream.add_module_license("BSD")
@@ -611,8 +634,9 @@ class TestModuleStream(TestBase):
         v1_stream.add_buildtime_requirement("ModuleB", "streamY")
         v1_stream.add_runtime_requirement("ModuleA", "streamZ")
         v1_stream.add_runtime_requirement("ModuleB", "streamY")
+        v1_stream.add_runtime_requirement("platform", "f33")
 
-        v2_stream = v1_stream.upgrade(Modulemd.ModuleStreamVersionEnum.LATEST)
+        v2_stream = v1_stream.upgrade(Modulemd.ModuleStreamVersionEnum.TWO)
         self.assertIsNotNone(v2_stream)
 
         idx = Modulemd.ModuleIndex.new()
@@ -625,7 +649,8 @@ document: modulemd
 version: 2
 data:
   name: SuperModule
-  stream: latest
+  stream: \"latest\"
+  context: ctx
   summary: Summary
   description: >-
     Description
@@ -639,6 +664,7 @@ data:
     requires:
       ModuleA: [streamZ]
       ModuleB: [streamY]
+      platform: [f33]
 ...
 """,
         )
@@ -650,7 +676,7 @@ document: modulemd
 version: 2
 data:
   name: modulename
-  stream: streamname
+  stream: \"streamname\"
   version: 1
   context: c0ffe3
   arch: x86_64
@@ -1204,71 +1230,72 @@ data:
                 )
 
     def test_validate_buildafter(self):
-        # buildafter is supported only on v2
-
-        # Test a valid module stream with buildafter set
-        stream = Modulemd.ModuleStream.read_file(
-            os.path.join(
-                self.test_data_path, "buildafter/good_buildafter.yaml"
-            ),
-            True,
-        )
-
-        self.assertIsNotNone(stream)
-        self.assertTrue(stream.validate())
-
-        # Should fail validation if both buildorder and buildafter are set for
-        # the same component.
-        with self.assertRaisesRegexp(
-            gi.repository.GLib.GError, "Cannot mix buildorder and buildafter"
-        ):
+        for version in modulestream_versions:
+            # buildafter is supported starting with v2
+            if version < Modulemd.ModuleStreamVersionEnum.TWO:
+                continue
+            # Test a valid module stream with buildafter set
             stream = Modulemd.ModuleStream.read_file(
-                os.path.join(
-                    self.test_data_path, "buildafter/both_same_component.yaml"
-                ),
+                "%s/buildafter/good_buildafter_v%d.yaml"
+                % (self.test_data_path, version),
                 True,
             )
+            self.assertIsNotNone(stream)
+            self.assertTrue(stream.validate())
 
-        # Should fail validation if both buildorder and buildafter are set in
-        # different components of the same stream.
-        with self.assertRaisesRegexp(
-            gi.repository.GLib.GError, "Cannot mix buildorder and buildafter"
-        ):
-            stream = Modulemd.ModuleStream.read_file(
-                os.path.join(
-                    self.test_data_path, "buildafter/mixed_buildorder.yaml"
-                ),
-                True,
-            )
+            # Should fail validation if both buildorder and buildafter are set for
+            # the same component.
+            with self.assertRaisesRegexp(
+                gi.repository.GLib.GError,
+                "Cannot mix buildorder and buildafter",
+            ):
+                stream = Modulemd.ModuleStream.read_file(
+                    "%s/buildafter/both_same_component_v%d.yaml"
+                    % (self.test_data_path, version),
+                    True,
+                )
 
-        # Should fail if a key specified in a buildafter set does not exist
-        # for this module stream.
-        with self.assertRaisesRegexp(
-            gi.repository.GLib.GError, "not found in components list"
-        ):
-            stream = Modulemd.ModuleStream.read_file(
-                os.path.join(
-                    self.test_data_path, "buildafter/invalid_key.yaml"
-                ),
-                True,
-            )
+            # Should fail validation if both buildorder and buildafter are set in
+            # different components of the same stream.
+            with self.assertRaisesRegexp(
+                gi.repository.GLib.GError,
+                "Cannot mix buildorder and buildafter",
+            ):
+                stream = Modulemd.ModuleStream.read_file(
+                    "%s/buildafter/mixed_buildorder_v%d.yaml"
+                    % (self.test_data_path, version),
+                    True,
+                )
+
+            # Should fail if a key specified in a buildafter set does not exist
+            # for this module stream.
+            with self.assertRaisesRegexp(
+                gi.repository.GLib.GError, "not found in components list"
+            ):
+                stream = Modulemd.ModuleStream.read_file(
+                    "%s/buildafter/invalid_key_v%d.yaml"
+                    % (self.test_data_path, version),
+                    True,
+                )
 
     def test_unicode_desc(self):
-        # Test a valid module stream with unicode in the description
-        stream = Modulemd.ModuleStream.read_file(
-            "%s/stream_unicode.yaml" % (os.getenv("TEST_DATA_PATH")),
-            True,
-            "",
-            "",
-        )
+        for version in modulestream_versions:
+            # Test a valid module stream with unicode in the description
+            stream = Modulemd.ModuleStream.read_file(
+                "%s/stream_unicode_v%d.yaml"
+                % (os.getenv("TEST_DATA_PATH"), version),
+                True,
+                "",
+                "",
+            )
 
-        self.assertIsNotNone(stream)
-        self.assertTrue(stream.validate())
+            self.assertIsNotNone(stream)
+            self.assertTrue(stream.validate())
 
     def test_xmd_issue_274(self):
         # Test a valid module stream with unicode in the description
         stream = Modulemd.ModuleStream.read_file(
-            "%s/stream_unicode.yaml" % (os.getenv("TEST_DATA_PATH")),
+            "%s/stream_unicode_v2.yaml" % (os.getenv("TEST_DATA_PATH")),
             True,
             "",
             "",
@@ -1399,6 +1426,108 @@ data:
         self.assertIsInstance(profiles[2], Modulemd.Profile)
         self.assertEqual(profiles[2].get_name(), "thethirdprofile")
 
+    def test_static_context(self):
+        stream = Modulemd.ModuleStreamV2.new("themodule", "thestream")
+
+        # Verify that it defaults to FALSE
+        self.assertFalse(stream.props.static_context)
+        self.assertFalse(stream.is_static_context())
+
+        # Test setting and unsetting it via methods
+        stream.set_static_context()
+        self.assertTrue(stream.props.static_context)
+        self.assertTrue(stream.is_static_context())
+
+        stream.unset_static_context()
+        self.assertFalse(stream.props.static_context)
+        self.assertFalse(stream.is_static_context())
+
+        # Test setting and unsetting it via properties
+        stream.props.static_context = True
+        self.assertTrue(stream.props.static_context)
+        self.assertTrue(stream.is_static_context())
+
+        stream.props.static_context = False
+        self.assertFalse(stream.props.static_context)
+        self.assertFalse(stream.is_static_context())
+
+        # Read in a stream with a static context
+        idx = Modulemd.load_file(
+            "%s/static_context.yaml" % (os.getenv("TEST_DATA_PATH"))
+        )
+        streams = idx.search_streams()
+        self.assertEquals(1, len(streams))
+        stream = streams[0]
+        self.assertTrue(stream.props.static_context)
+        self.assertTrue(stream.is_static_context())
+
+        expected = """---
+document: modulemd
+version: 2
+data:
+  name: nodejs
+  stream: "8"
+  version: 20180816123422
+  context: RealCTX
+  static_context: true
+  arch: x86_64
+  summary: Javascript runtime
+  description: >-
+    Node.js is a platform built on Chrome''s JavaScript runtime for easily building
+    fast, scalable network applications. Node.js uses an event-driven, non-blocking
+    I/O model that makes it lightweight and efficient, perfect for data-intensive
+    real-time applications that run across distributed devices.
+  license:
+    module:
+    - MIT
+    content:
+    - MIT and ASL 2.0 and ISC and BSD
+  dependencies:
+  - buildrequires:
+      platform: [f29]
+    requires:
+      platform: [f29]
+  references:
+    community: http://nodejs.org
+    documentation: http://nodejs.org/en/docs
+    tracker: https://github.com/nodejs/node/issues
+  profiles:
+    default:
+      rpms:
+      - nodejs
+      - npm
+    development:
+      rpms:
+      - nodejs
+      - nodejs-devel
+      - npm
+    minimal:
+      rpms:
+      - nodejs
+  api:
+    rpms:
+    - nodejs
+    - nodejs-devel
+    - npm
+  components:
+    rpms:
+      nodejs:
+        rationale: Javascript runtime and npm package manager.
+        repository: git://pkgs.fedoraproject.org/rpms/nodejs
+        cache: http://pkgs.fedoraproject.org/repo/pkgs/nodejs
+        ref: 8
+        buildorder: 10
+  artifacts:
+    rpms:
+    - nodejs-1:8.11.4-1.module_2030+42747d40.x86_64
+    - nodejs-devel-1:8.11.4-1.module_2030+42747d40.x86_64
+    - nodejs-docs-1:8.11.4-1.module_2030+42747d40.noarch
+    - npm-1:5.6.0-1.8.11.4.1.module_2030+42747d40.x86_64
+...
+"""
+        self.maxDiff = None
+        self.assertEquals(expected, idx.dump_to_string())
+
 
 if __name__ == "__main__":
     unittest.main()
