@@ -1,6 +1,6 @@
 /*
  * This file is part of libmodulemd
- * Copyright (C) 2018 Red Hat, Inc.
+ * Copyright (C) 2018-2020 Red Hat, Inc.
  *
  * Fedora-License-Identifier: MIT
  * SPDX-2.0-License-Identifier: MIT
@@ -25,6 +25,9 @@
 #include "private/glib-extensions.h"
 #include "private/modulemd-defaults-v1-private.h"
 #include "private/modulemd-module-private.h"
+#include "private/modulemd-module-stream-private.h"
+#include "private/modulemd-obsoletes-private.h"
+#include "private/modulemd-subdocument-info-private.h"
 #include "private/modulemd-util.h"
 #include "private/modulemd-yaml.h"
 #include "private/test-utils.h"
@@ -58,7 +61,7 @@ module_test_construct (void)
   g_clear_error (&error);
   g_clear_object (&m);
 
-  /* Test that object instantiation works wiht a name */
+  /* Test that object instantiation works with a name */
   m = g_object_new (MODULEMD_TYPE_MODULE, "module-name", "testmodule", NULL);
   g_assert_true (MODULEMD_IS_MODULE (m));
   g_assert_cmpstr (modulemd_module_get_module_name (m), ==, "testmodule");
@@ -139,6 +142,7 @@ module_test_streams (void)
   g_autoptr (ModulemdModule) m = modulemd_module_new ("testmodule");
   g_autoptr (ModulemdTranslation) t = NULL;
   g_autoptr (ModulemdTranslationEntry) te = NULL;
+  ModulemdObsoletes *o = NULL;
   g_autoptr (GError) error = NULL;
   ModulemdModuleStream *stream = NULL;
   GPtrArray *list = NULL;
@@ -152,46 +156,53 @@ module_test_streams (void)
   modulemd_module_add_translation (m, t);
   g_clear_pointer (&t, g_object_unref);
 
+  /* Create an obsoletes pre-adding streams */
+  o = modulemd_obsoletes_new (
+    1, 2, "testmodule", "stream1", "obsolete1 added to context1");
+  modulemd_obsoletes_set_module_context (o, "context1");
+  modulemd_module_add_obsoletes (m, o);
+  g_clear_pointer (&o, g_object_unref);
+
   /* Create and add some streams to cross */
   stream = modulemd_module_stream_new (2, "testmodule", "stream1");
   modulemd_module_stream_set_version (stream, 1);
   modulemd_module_stream_set_context (stream, "context1");
   modulemd_module_stream_v2_set_summary (MODULEMD_MODULE_STREAM_V2 (stream),
                                          "Stream 1");
-  g_assert_cmpint (modulemd_module_add_stream (
-                     m, stream, MD_MODULESTREAM_VERSION_UNSET, NULL),
-                   ==,
-                   MD_MODULESTREAM_VERSION_TWO);
+  g_assert_cmpint (
+    modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_TWO, NULL),
+    ==,
+    MD_MODULESTREAM_VERSION_TWO);
   g_clear_object (&stream);
   stream = modulemd_module_stream_new (2, "testmodule", "stream1");
   modulemd_module_stream_set_version (stream, 3);
   modulemd_module_stream_set_context (stream, "context2");
   modulemd_module_stream_v2_set_summary (MODULEMD_MODULE_STREAM_V2 (stream),
                                          "Stream 1");
-  g_assert_cmpint (modulemd_module_add_stream (
-                     m, stream, MD_MODULESTREAM_VERSION_UNSET, NULL),
-                   ==,
-                   MD_MODULESTREAM_VERSION_TWO);
+  g_assert_cmpint (
+    modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_TWO, NULL),
+    ==,
+    MD_MODULESTREAM_VERSION_TWO);
   g_clear_object (&stream);
   stream = modulemd_module_stream_new (2, "testmodule", "stream1");
   modulemd_module_stream_set_version (stream, 1);
   modulemd_module_stream_set_context (stream, "context2");
   modulemd_module_stream_v2_set_summary (MODULEMD_MODULE_STREAM_V2 (stream),
                                          "Stream 1");
-  g_assert_cmpint (modulemd_module_add_stream (
-                     m, stream, MD_MODULESTREAM_VERSION_UNSET, NULL),
-                   ==,
-                   MD_MODULESTREAM_VERSION_TWO);
+  g_assert_cmpint (
+    modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_TWO, NULL),
+    ==,
+    MD_MODULESTREAM_VERSION_TWO);
   g_clear_object (&stream);
   stream = modulemd_module_stream_new (2, "testmodule", "stream2");
   modulemd_module_stream_set_version (stream, 42);
   modulemd_module_stream_set_context (stream, "context42");
   modulemd_module_stream_v2_set_summary (MODULEMD_MODULE_STREAM_V2 (stream),
                                          "Stream 2");
-  g_assert_cmpint (modulemd_module_add_stream (
-                     m, stream, MD_MODULESTREAM_VERSION_UNSET, NULL),
-                   ==,
-                   MD_MODULESTREAM_VERSION_TWO);
+  g_assert_cmpint (
+    modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_TWO, NULL),
+    ==,
+    MD_MODULESTREAM_VERSION_TWO);
   g_clear_object (&stream);
 
   /* Create a translation post-adding streams */
@@ -203,6 +214,12 @@ module_test_streams (void)
   modulemd_module_add_translation (m, t);
   g_clear_pointer (&t, g_object_unref);
 
+  /* Create an obsoletes post-adding streams */
+  o = modulemd_obsoletes_new (
+    1, 2, "testmodule", "stream1", "obsolete2 added to all stream");
+  modulemd_module_add_obsoletes (m, o);
+  g_clear_pointer (&o, g_object_unref);
+
   /* Verify that we get all streams */
   list = modulemd_module_get_all_streams (m);
   g_assert_cmpint (list->len, ==, 4);
@@ -279,6 +296,11 @@ module_test_streams (void)
                      MODULEMD_MODULE_STREAM_V2 (stream), "nl_NL"),
                    ==,
                    "Een test omschrijving");
+  o = modulemd_module_stream_v2_get_obsoletes_resolved (
+    (ModulemdModuleStreamV2 *)stream);
+  g_assert_nonnull (o);
+  g_assert_cmpstr (
+    modulemd_obsoletes_get_message (o), ==, "obsolete1 added to context1");
 
   stream = modulemd_module_get_stream_by_NSVCA (
     m, "stream1", 1, "context2", NULL, &error);
@@ -290,6 +312,11 @@ module_test_streams (void)
   g_assert_cmpint (modulemd_module_stream_get_version (stream), ==, 1);
   g_assert_cmpstr (
     modulemd_module_stream_get_context (stream), ==, "context2");
+  o = modulemd_module_stream_v2_get_obsoletes_resolved (
+    (ModulemdModuleStreamV2 *)stream);
+  g_assert_nonnull (o);
+  g_assert_cmpstr (
+    modulemd_obsoletes_get_message (o), ==, "obsolete2 added to all stream");
 
   stream = modulemd_module_get_stream_by_NSVCA (
     m, "stream1", 3, "context1", NULL, &error);
@@ -301,6 +328,11 @@ module_test_streams (void)
     m, "stream1", 3, "context2", NULL, &error);
   g_assert_nonnull (stream);
   g_assert_no_error (error);
+  o = modulemd_module_stream_v2_get_obsoletes_resolved (
+    (ModulemdModuleStreamV2 *)stream);
+  g_assert_nonnull (o);
+  g_assert_cmpstr (
+    modulemd_obsoletes_get_message (o), ==, "obsolete2 added to all stream");
 
   g_assert_cmpstr (
     modulemd_module_stream_get_stream_name (stream), ==, "stream1");
@@ -346,11 +378,13 @@ module_test_get_stream_names (void)
 
   /*Test module with all same stream names*/
   m = modulemd_module_new ("testmodule");
-  stream = modulemd_module_stream_new (2, "testmodule", "stream1");
-  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_UNSET, NULL);
+  stream = modulemd_module_stream_new (
+    MD_MODULESTREAM_VERSION_TWO, "testmodule", "stream1");
+  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_TWO, NULL);
   g_clear_object (&stream);
-  stream = modulemd_module_stream_new (2, "testmodule", "stream1");
-  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_UNSET, NULL);
+  stream = modulemd_module_stream_new (
+    MD_MODULESTREAM_VERSION_TWO, "testmodule", "stream1");
+  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_TWO, NULL);
   g_clear_object (&stream);
 
   list = modulemd_module_get_stream_names_as_strv (m);
@@ -362,14 +396,17 @@ module_test_get_stream_names (void)
 
   /*Test module with all different stream names*/
   m = modulemd_module_new ("testmodule");
-  stream = modulemd_module_stream_new (2, "testmodule", "stream1");
-  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_UNSET, NULL);
+  stream = modulemd_module_stream_new (
+    MD_MODULESTREAM_VERSION_TWO, "testmodule", "stream1");
+  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_TWO, NULL);
   g_clear_object (&stream);
-  stream = modulemd_module_stream_new (2, "testmodule", "stream2");
-  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_UNSET, NULL);
+  stream = modulemd_module_stream_new (
+    MD_MODULESTREAM_VERSION_TWO, "testmodule", "stream2");
+  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_TWO, NULL);
   g_clear_object (&stream);
-  stream = modulemd_module_stream_new (2, "testmodule", "stream3");
-  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_UNSET, NULL);
+  stream = modulemd_module_stream_new (
+    MD_MODULESTREAM_VERSION_TWO, "testmodule", "stream3");
+  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_TWO, NULL);
   g_clear_object (&stream);
 
   list = modulemd_module_get_stream_names_as_strv (m);
@@ -385,14 +422,17 @@ module_test_get_stream_names (void)
 
   /*Test module with some same/different stream names*/
   m = modulemd_module_new ("testmodule");
-  stream = modulemd_module_stream_new (2, "testmodule", "stream1");
-  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_UNSET, NULL);
+  stream = modulemd_module_stream_new (
+    MD_MODULESTREAM_VERSION_TWO, "testmodule", "stream1");
+  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_TWO, NULL);
   g_clear_object (&stream);
-  stream = modulemd_module_stream_new (2, "testmodule", "stream1");
-  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_UNSET, NULL);
+  stream = modulemd_module_stream_new (
+    MD_MODULESTREAM_VERSION_TWO, "testmodule", "stream1");
+  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_TWO, NULL);
   g_clear_object (&stream);
-  stream = modulemd_module_stream_new (2, "testmodule", "stream2");
-  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_UNSET, NULL);
+  stream = modulemd_module_stream_new (
+    MD_MODULESTREAM_VERSION_TWO, "testmodule", "stream2");
+  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_TWO, NULL);
   g_clear_object (&stream);
 
   list = modulemd_module_get_stream_names_as_strv (m);
@@ -418,15 +458,18 @@ modulemd_test_remove_streams (void)
   g_autoptr (GError) error = NULL;
   g_autoptr (GPtrArray) failures = NULL;
   g_autofree gchar *yaml_path = NULL;
+  gboolean ret;
 
   /* Get the f29 and f29-updates indexes. They have multiple streams and
    * versions for the 'dwm' module
    */
   f29 = modulemd_module_index_new ();
   yaml_path = g_strdup_printf ("%s/f29.yaml", g_getenv ("TEST_DATA_PATH"));
-  g_assert_true (modulemd_module_index_update_from_file (
-    f29, yaml_path, TRUE, &failures, &error));
+  ret = modulemd_module_index_update_from_file (
+    f29, yaml_path, TRUE, &failures, &error);
+  modulemd_subdocument_info_debug_dump_failures (failures);
   g_assert_no_error (error);
+  g_assert_true (ret);
   g_assert_cmpint (failures->len, ==, 0);
   g_clear_pointer (&yaml_path, g_free);
   g_clear_pointer (&failures, g_ptr_array_unref);
@@ -434,9 +477,10 @@ modulemd_test_remove_streams (void)
   f29_updates = modulemd_module_index_new ();
   yaml_path =
     g_strdup_printf ("%s/f29-updates.yaml", g_getenv ("TEST_DATA_PATH"));
-  g_assert_true (modulemd_module_index_update_from_file (
-    f29_updates, yaml_path, TRUE, &failures, &error));
+  ret = modulemd_module_index_update_from_file (
+    f29_updates, yaml_path, TRUE, &failures, &error);
   g_assert_no_error (error);
+  g_assert_true (ret);
   g_assert_cmpint (failures->len, ==, 0);
   g_clear_pointer (&yaml_path, g_free);
   g_clear_pointer (&failures, g_ptr_array_unref);
@@ -493,6 +537,7 @@ modulemd_test_remove_streams (void)
 static void
 module_test_search_streams_by_glob (void)
 {
+  gboolean ret;
   g_autoptr (ModulemdModuleIndex) index = modulemd_module_index_new ();
   g_autoptr (GError) error = NULL;
   g_autoptr (GPtrArray) failures = NULL;
@@ -503,9 +548,10 @@ module_test_search_streams_by_glob (void)
   yaml_path = g_strdup_printf ("%s/search_streams/search_streams.yaml",
                                g_getenv ("TEST_DATA_PATH"));
 
-  modulemd_module_index_update_from_file (
+  ret = modulemd_module_index_update_from_file (
     index, yaml_path, TRUE, &failures, &error);
   g_assert_no_error (error);
+  g_assert_true (ret);
 
   module = modulemd_module_index_get_module (index, "nodejs");
   g_assert_nonnull (module);
@@ -569,6 +615,7 @@ module_test_search_streams_by_glob (void)
 static void
 module_test_search_streams_by_nsvca_glob (void)
 {
+  gboolean ret;
   g_autoptr (ModulemdModuleIndex) index = modulemd_module_index_new ();
   g_autoptr (GError) error = NULL;
   g_autoptr (GPtrArray) failures = NULL;
@@ -579,9 +626,10 @@ module_test_search_streams_by_nsvca_glob (void)
   yaml_path = g_strdup_printf ("%s/search_streams/search_streams.yaml",
                                g_getenv ("TEST_DATA_PATH"));
 
-  modulemd_module_index_update_from_file (
+  ret = modulemd_module_index_update_from_file (
     index, yaml_path, TRUE, &failures, &error);
   g_assert_no_error (error);
+  g_assert_true (ret);
 
   module = modulemd_module_index_get_module (index, "nodejs");
   g_assert_nonnull (module);
@@ -612,6 +660,119 @@ module_test_search_streams_by_nsvca_glob (void)
   g_clear_pointer (&streams, g_ptr_array_unref);
 }
 
+static void
+module_test_get_newest_active_obsoletes (void)
+{
+  ModulemdModule *m = NULL;
+  ModulemdObsoletes *o = NULL;
+
+  m = modulemd_module_new ("testmodule");
+
+  o = modulemd_obsoletes_new (
+    1, 3, "testmodule", "stream1", "The newest active obsolete");
+  modulemd_obsoletes_set_eol_date (o, 201807011200);
+  modulemd_module_add_obsoletes (m, o);
+  g_clear_pointer (&o, g_object_unref);
+
+  o = modulemd_obsoletes_new (
+    1, 1, "testmodule", "stream1", "obsolete2 added to all stream");
+  modulemd_obsoletes_set_eol_date (o, 2);
+  modulemd_module_add_obsoletes (m, o);
+  g_clear_pointer (&o, g_object_unref);
+
+  o = modulemd_obsoletes_new (
+    1, 1, "testmodule", "stream1", "obsolete3 added to all stream");
+  modulemd_obsoletes_set_eol_date (o, 291807011200);
+  modulemd_module_add_obsoletes (m, o);
+  g_clear_pointer (&o, g_object_unref);
+
+  o = modulemd_module_get_newest_active_obsoletes (m, "stream1", NULL);
+  g_assert_nonnull (o);
+  g_assert_cmpstr (
+    modulemd_obsoletes_get_message (o), ==, "The newest active obsolete");
+
+  g_clear_object (&m);
+}
+
+static void
+module_test_get_obsoletes (void)
+{
+  ModulemdModule *m = NULL;
+  ModulemdModuleStream *stream = NULL;
+  ModulemdObsoletes *o = NULL;
+
+  m = modulemd_module_new ("testmodule");
+
+  stream = modulemd_module_stream_new (2, "testmodule", "stream1");
+  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_UNSET, NULL);
+  g_clear_object (&stream);
+  stream = modulemd_module_stream_new (2, "testmodule", "stream2");
+  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_UNSET, NULL);
+  g_clear_object (&stream);
+  stream = modulemd_module_stream_new (2, "testmodule", "stream3");
+  modulemd_module_add_stream (m, stream, MD_MODULESTREAM_VERSION_UNSET, NULL);
+  g_clear_object (&stream);
+
+  o = modulemd_obsoletes_new (
+    1, 2, "testmodule", "stream1", "obsolete1 added to all stream1");
+  modulemd_module_add_obsoletes (m, o);
+  g_clear_pointer (&o, g_object_unref);
+
+  o = modulemd_obsoletes_new (
+    1, 3, "testmodule", "stream2", "obsolete2 added to all stream2");
+  modulemd_module_add_obsoletes (m, o);
+  g_clear_pointer (&o, g_object_unref);
+
+  o = modulemd_obsoletes_new (
+    1, 3, "testmodule", "stream2", "obsolete3 added to all stream2");
+  modulemd_obsoletes_set_module_context (o, "context");
+  modulemd_module_add_obsoletes (m, o);
+  g_clear_pointer (&o, g_object_unref);
+
+
+  GPtrArray *obsoletes = modulemd_module_get_obsoletes (m);
+  g_assert_cmpint (obsoletes->len, ==, 3);
+
+  g_clear_object (&m);
+}
+
+static void
+module_test_add_stream_to_module_with_obsoletes (void)
+{
+  ModulemdModule *m = NULL;
+  GPtrArray *streams = NULL;
+  ModulemdModuleStream *s = NULL;
+  ModulemdObsoletes *o = NULL;
+
+  m = modulemd_module_new ("nodejs");
+  g_assert_nonnull (m);
+  o = modulemd_obsoletes_new (1, 3, "nodejs", "8.0", "test message");
+  modulemd_obsoletes_set_module_context (o, "42");
+  modulemd_obsoletes_set_obsoleted_by (o, "nodejs", "12");
+  modulemd_module_add_obsoletes (m, o);
+  g_clear_pointer (&o, g_object_unref);
+
+  s =
+    modulemd_module_stream_new (MD_MODULESTREAM_VERSION_TWO, "nodejs", "8.0");
+  modulemd_module_stream_set_context (s, "42");
+  modulemd_module_add_stream (m, s, MD_MODULESTREAM_VERSION_TWO, NULL);
+  g_clear_pointer (&s, g_object_unref);
+
+  streams = modulemd_module_get_all_streams (m);
+  g_assert_cmpint (streams->len, ==, 1);
+  s = g_ptr_array_index (streams, 0);
+  g_assert_nonnull (s);
+
+  o = modulemd_module_stream_v2_get_obsoletes_resolved (
+    (ModulemdModuleStreamV2 *)s);
+  g_assert_nonnull (o);
+  g_assert_cmpstr (
+    modulemd_obsoletes_get_obsoleted_by_module_name (o), ==, "nodejs");
+  g_assert_cmpstr (
+    modulemd_obsoletes_get_obsoleted_by_module_stream (o), ==, "12");
+
+  g_clear_pointer (&m, g_object_unref);
+}
 
 int
 main (int argc, char *argv[])
@@ -641,5 +802,15 @@ main (int argc, char *argv[])
   g_test_add_func ("/modulemd/v2/module/streams/glob_nsvca",
                    module_test_search_streams_by_nsvca_glob);
 
+  g_test_add_func ("/modulemd/v2/module/obsoletes/get_newest_active_obsoletes",
+                   module_test_get_newest_active_obsoletes);
+
+  g_test_add_func ("/modulemd/v2/module/obsoletes/get_obsoletes",
+                   module_test_get_obsoletes);
+
+  g_test_add_func (
+    "/modulemd/v2/module/obsoletes/add_stream_to_module_with_obsoletes",
+    module_test_add_stream_to_module_with_obsoletes);
+
   return g_test_run ();
 }
